@@ -69,6 +69,11 @@ G_DEFINE_TYPE (GstGLWindow, gst_gl_window, G_TYPE_OBJECT);
 
 gboolean _gst_gl_window_debug = FALSE;
 
+void
+gst_gl_window_init_platform ()
+{
+}
+
 HHOOK hHook;
 
 /* Must be called in the gl thread */
@@ -136,7 +141,7 @@ gst_gl_window_init (GstGLWindow * window)
 
 /* Must be called in the gl thread */
 GstGLWindow *
-gst_gl_window_new (gint width, gint height, DWORD_PTR external_gl_context)
+gst_gl_window_new (DWORD_PTR external_gl_context)
 {
   GstGLWindow *window = g_object_new (GST_GL_TYPE_WINDOW, NULL);
   GstGLWindowPrivate *priv = window->priv;
@@ -162,29 +167,32 @@ gst_gl_window_new (gint width, gint height, DWORD_PTR external_gl_context)
   priv->is_closed = FALSE;
   priv->visible = FALSE;
 
-  width += 2 * GetSystemMetrics (SM_CXSIZEFRAME);
-  height +=
-      2 * GetSystemMetrics (SM_CYSIZEFRAME) + GetSystemMetrics (SM_CYCAPTION);
-
   priv->internal_win_id = CreateWindowEx (0,
       "GSTGL",
       "OpenGL renderer",
       WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_OVERLAPPEDWINDOW,
-      x, y, width, height, (HWND) NULL, (HMENU) NULL, hinstance, window);
+      x, y, 0, 0, (HWND) NULL, (HMENU) NULL, hinstance, window);
 
   if (!priv->internal_win_id) {
-    g_debug ("failed to create gl window: %lud\n", (gulong) priv->internal_win_id);
-    return NULL;
+    g_debug ("failed to create gl window\n");
+    goto failure;
   }
 
   g_debug ("gl window created: %lud\n", (gulong) priv->internal_win_id);
 
   //device is set in the window_proc
-  g_assert (priv->device);
+  if (!priv->device) {
+    g_debug ("failed to create device\n");
+    goto failure;
+  }
 
   ShowCursor (TRUE);
 
   return window;
+
+failure:
+  g_object_unref (G_OBJECT (window));
+  return NULL;
 }
 
 GQuark
@@ -193,6 +201,39 @@ gst_gl_window_error_quark (void)
   return g_quark_from_static_string ("gst-gl-window-error");
 }
 
+gulong
+gst_gl_window_get_internal_gl_context (GstGLWindow * window)
+{
+  GstGLWindowPrivate *priv = window->priv;
+  return (gulong) priv->gl_context;
+}
+
+void
+callback_activate_gl_context (GstGLWindowPrivate * priv)
+{
+  if (!wglMakeCurrent (priv->device, priv->gl_context))
+    g_debug ("failed to activate opengl context %lud\n", GetLastError ());
+}
+
+void
+callback_inactivate_gl_context (GstGLWindowPrivate * priv)
+{
+  if (!wglMakeCurrent (NULL, NULL))
+    g_debug ("failed to inactivate opengl context %lud\n", GetLastError ());
+}
+
+void
+gst_gl_window_activate_gl_context (GstGLWindow * window, gboolean activate)
+{
+  GstGLWindowPrivate *priv = window->priv;
+  if (activate)
+    gst_gl_window_send_message (window,
+        GST_GL_WINDOW_CB (callback_activate_gl_context), priv);
+  else
+    gst_gl_window_send_message (window,
+        GST_GL_WINDOW_CB (callback_inactivate_gl_context), priv);
+}
+
 void
 gst_gl_window_set_external_window_id (GstGLWindow * window, gulong id)
 {
@@ -202,16 +243,16 @@ gst_gl_window_set_external_window_id (GstGLWindow * window, gulong id)
   HWND parent_id = GetProp (priv->internal_win_id, "gl_window_parent_id");
 
   if (priv->visible) {
-      ShowWindow (priv->internal_win_id, SW_HIDE);
-      priv->visible = FALSE;
-   }
-  
+    ShowWindow (priv->internal_win_id, SW_HIDE);
+    priv->visible = FALSE;
+  }
+
   if (parent_id) {
     WNDPROC parent_proc = GetProp (parent_id, "gl_window_parent_proc");
 
     g_debug ("release parent %lud\n", (gulong) parent_id);
 
-    g_assert (parent_proc);
+    g_return_if_fail (parent_proc);
 
     SetWindowLongPtr (parent_id, GWL_WNDPROC, (LONG) parent_proc);
     SetParent (priv->internal_win_id, NULL);
@@ -219,7 +260,6 @@ gst_gl_window_set_external_window_id (GstGLWindow * window, gulong id)
     RemoveProp (parent_id, "gl_window_parent_proc");
     RemoveProp (priv->internal_win_id, "gl_window_parent_id");
   }
-
   //not 0
   if (id) {
     WNDPROC window_parent_proc =
@@ -247,7 +287,7 @@ gst_gl_window_set_external_window_id (GstGLWindow * window, gulong id)
   } else {
     //no parent so the internal window needs borders and system menu
     SetWindowLongPtr (priv->internal_win_id, GWL_STYLE,
-      WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_OVERLAPPEDWINDOW);
+        WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_OVERLAPPEDWINDOW);
   }
 }
 
@@ -285,19 +325,33 @@ gst_gl_window_set_close_callback (GstGLWindow * window, GstGLWindowCB callback,
 }
 
 void
-gst_gl_window_draw_unlocked (GstGLWindow * window)
+gst_gl_window_draw_unlocked (GstGLWindow * window, gint width, gint height)
 {
-  gst_gl_window_draw (window);
+  gst_gl_window_draw (window, width, height);
 }
 
 /* Thread safe */
 void
-gst_gl_window_draw (GstGLWindow * window)
+gst_gl_window_draw (GstGLWindow * window, gint width, gint height)
 {
   GstGLWindowPrivate *priv = window->priv;
 
   if (!priv->visible) {
+    HWND parent_id = GetProp (priv->internal_win_id, "gl_window_parent_id");
+    /* if no parent the real size has to be set now because this has not been done
+     * when at window creation */
+    if (!parent_id) {
+      RECT rect;
+      GetClientRect (priv->internal_win_id, &rect);
+      width += 2 * GetSystemMetrics (SM_CXSIZEFRAME);
+      height +=
+          2 * GetSystemMetrics (SM_CYSIZEFRAME) +
+          GetSystemMetrics (SM_CYCAPTION);
+      MoveWindow (priv->internal_win_id, rect.left, rect.top, width, height,
+          FALSE);
+    }
     ShowWindowAsync (priv->internal_win_id, SW_SHOW);
+
     priv->visible = TRUE;
   }
 
@@ -336,7 +390,7 @@ gst_gl_window_quit_loop (GstGLWindow * window, GstGLWindowCB callback,
     GstGLWindowPrivate *priv = window->priv;
     LRESULT res = PostMessage (priv->internal_win_id, WM_GST_GL_WINDOW_QUIT,
         (WPARAM) data, (LPARAM) callback);
-    g_assert (SUCCEEDED (res));
+    g_return_if_fail (SUCCEEDED (res));
     g_debug ("end loop requested\n");
   }
 }
@@ -350,7 +404,7 @@ gst_gl_window_send_message (GstGLWindow * window, GstGLWindowCB callback,
     GstGLWindowPrivate *priv = window->priv;
     LRESULT res = SendMessage (priv->internal_win_id, WM_GST_GL_WINDOW_CUSTOM,
         (WPARAM) data, (LPARAM) callback);
-    g_assert (SUCCEEDED (res));
+    g_return_if_fail (SUCCEEDED (res));
   }
 }
 
@@ -369,11 +423,11 @@ gst_gl_window_set_pixel_format (GstGLWindow * window)
   pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
   pfd.iPixelType = PFD_TYPE_RGBA;
   pfd.cColorBits = 24;
-  pfd.cRedBits = 0;
+  pfd.cRedBits = 8;
   pfd.cRedShift = 0;
-  pfd.cGreenBits = 0;
+  pfd.cGreenBits = 8;
   pfd.cGreenShift = 0;
-  pfd.cBlueBits = 0;
+  pfd.cBlueBits = 8;
   pfd.cBlueShift = 0;
   pfd.cAlphaBits = 0;
   pfd.cAlphaShift = 0;
@@ -382,7 +436,7 @@ gst_gl_window_set_pixel_format (GstGLWindow * window)
   pfd.cAccumGreenBits = 0;
   pfd.cAccumBlueBits = 0;
   pfd.cAccumAlphaBits = 0;
-  pfd.cDepthBits = 32;
+  pfd.cDepthBits = 24;
   pfd.cStencilBits = 8;
   pfd.cAuxBuffers = 0;
   pfd.iLayerType = PFD_MAIN_PLANE;
@@ -395,11 +449,11 @@ gst_gl_window_set_pixel_format (GstGLWindow * window)
 
   pixelformat = ChoosePixelFormat (priv->device, &pfd);
 
-  g_assert (pixelformat);
+  g_return_if_fail (pixelformat);
 
   res = SetPixelFormat (priv->device, pixelformat, &pfd);
 
-  g_assert (res);
+  g_return_if_fail (res);
 }
 
 LRESULT CALLBACK
@@ -427,15 +481,16 @@ window_proc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
       g_assert (priv->gl_context);
       ReleaseDC (hWnd, priv->device);
       if (!wglMakeCurrent (priv->device, priv->gl_context))
-        g_debug ("failed to make opengl context current %lud, %lud\n", (gulong) hWnd,
-            GetLastError ());
+        g_debug ("failed to make opengl context current %lud, %lud\n",
+            (gulong) hWnd, GetLastError ());
 
       if (priv->external_gl_context) {
-        if (!wglShareLists (priv->gl_context, priv->external_gl_context))
+        if (!wglShareLists (priv->external_gl_context, priv->gl_context))
           g_debug ("failed to share opengl context %lud with %lud\n",
               (gulong) priv->gl_context, (gulong) priv->external_gl_context);
         else
-          g_debug ("share opengl context succeed %lud\n", (gulong) priv->external_gl_context);
+          g_debug ("share opengl context succeed %lud\n",
+              (gulong) priv->external_gl_context);
       }
     }
 
@@ -455,14 +510,16 @@ window_proc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 
     g_assert (priv->internal_win_id == hWnd);
 
-    g_assert (priv->gl_context == wglGetCurrentContext ());
+    g_assert (!wglGetCurrentContext ()
+        || priv->gl_context == wglGetCurrentContext ());
 
     switch (uMsg) {
 
       case WM_SIZE:
       {
-        if (priv->resize_cb)
+        if (priv->resize_cb) {
           priv->resize_cb (priv->resize_data, LOWORD (lParam), HIWORD (lParam));
+        }
         break;
       }
 
@@ -509,8 +566,7 @@ window_proc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 
           g_assert (parent_proc);
 
-          SetWindowLongPtr (parent_id, GWL_WNDPROC,
-              (LONG) parent_proc);
+          SetWindowLongPtr (parent_id, GWL_WNDPROC, (LONG) parent_proc);
           SetParent (hWnd, NULL);
 
           RemoveProp (parent_id, "gl_window_parent_proc");
@@ -521,12 +577,13 @@ window_proc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
         RemoveProp (hWnd, "gl_window");
 
         if (!wglMakeCurrent (NULL, NULL))
-          g_debug ("failed to make current %lud, %lud\n", (gulong) hWnd, GetLastError ());
+          g_debug ("failed to make current %lud, %lud\n", (gulong) hWnd,
+              GetLastError ());
 
         if (priv->gl_context) {
           if (!wglDeleteContext (priv->gl_context))
-            g_debug ("failed to destroy context %lud, %lud\n", (gulong) priv->gl_context,
-                GetLastError ());
+            g_debug ("failed to destroy context %lud, %lud\n",
+                (gulong) priv->gl_context, GetLastError ());
         }
 
         if (priv->internal_win_id) {
@@ -560,13 +617,13 @@ window_proc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
         return TRUE;
 
       default:
-        {
-          /* transmit messages to the parrent (ex: mouse/keyboard input) */
-          HWND parent_id = GetProp (hWnd, "gl_window_parent_id");
-          if (parent_id)
-            PostMessage (parent_id, uMsg, wParam, lParam);
-          return DefWindowProc (hWnd, uMsg, wParam, lParam);
-        }
+      {
+        /* transmit messages to the parrent (ex: mouse/keyboard input) */
+        HWND parent_id = GetProp (hWnd, "gl_window_parent_id");
+        if (parent_id)
+          PostMessage (parent_id, uMsg, wParam, lParam);
+        return DefWindowProc (hWnd, uMsg, wParam, lParam);
+      }
     }
 
     return 0;
