@@ -90,6 +90,11 @@ G_DEFINE_TYPE (GstGLWindow, gst_gl_window, G_TYPE_OBJECT);
 
 gboolean _gst_gl_window_debug = FALSE;
 
+void
+gst_gl_window_init_platform ()
+{
+}
+
 /* Must be called in the gl thread */
 static void
 gst_gl_window_finalize (GObject * object)
@@ -239,7 +244,7 @@ gst_gl_window_init (GstGLWindow * window)
 
 /* Must be called in the gl thread */
 GstGLWindow *
-gst_gl_window_new (gint width, gint height, gulong external_gl_context)
+gst_gl_window_new (gulong external_gl_context)
 {
   GstGLWindow *window = g_object_new (GST_GL_TYPE_WINDOW, NULL);
   GstGLWindowPrivate *priv = window->priv;
@@ -247,6 +252,7 @@ gst_gl_window_new (gint width, gint height, gulong external_gl_context)
   gint attrib[] = {
     GLX_RGBA,
     GLX_RED_SIZE, 1, GLX_GREEN_SIZE, 1, GLX_BLUE_SIZE, 1,
+    GLX_DEPTH_SIZE, 16,
     GLX_DOUBLEBUFFER,
     None
   };
@@ -277,8 +283,10 @@ gst_gl_window_new (gint width, gint height, gulong external_gl_context)
   g_mutex_lock (priv->x_lock);
 
   priv->device = XOpenDisplay (priv->display_name);
-  if (priv->device == NULL)
-    goto no_display;
+  if (priv->device == NULL) {
+    g_debug ("XOpenDisplay failed\n");
+    goto failure;
+  }
 
   XSynchronize (priv->device, FALSE);
 
@@ -306,14 +314,16 @@ gst_gl_window_new (gint width, gint height, gulong external_gl_context)
   priv->connection = ConnectionNumber (priv->device);
 
   ret = glXQueryExtension (priv->device, &error_base, &event_base);
-  if (!ret)
-    g_debug ("No GLX extension");
+  if (!ret) {
+    g_debug ("No GLX extension\n");
+    goto failure;
+  }
 
   priv->visual_info = glXChooseVisual (priv->device, priv->screen_num, attrib);
 
   if (!priv->visual_info) {
-    g_warning ("glx visual is null (bad attributes)\n");
-    return NULL;
+    g_debug ("glx visual is null (bad attributes)\n");
+    goto failure;
   }
 
   if (priv->visual_info->visual != priv->visual)
@@ -347,20 +357,20 @@ gst_gl_window_new (gint width, gint height, gulong external_gl_context)
 
   mask = CWBackPixmap | CWBorderPixel | CWColormap | CWEventMask;
 
-  priv->internal_win_id = XCreateWindow (priv->device, priv->root, x, y,
-      width, height, 0, priv->visual_info->depth, InputOutput,
-      priv->visual_info->visual, mask, &win_attr);
-
   x += 20;
   y += 20;
 
+  priv->internal_win_id = XCreateWindow (priv->device, priv->root, x, y,
+      1, 1, 0, priv->visual_info->depth, InputOutput,
+      priv->visual_info->visual, mask, &win_attr);
+
   XSync (priv->device, FALSE);
 
   XSetWindowBackgroundPixmap (priv->device, priv->internal_win_id, None);
 
   g_debug ("gl window id: %lud\n", (gulong) priv->internal_win_id);
 
-  g_debug ("gl window props: x:%d y:%d w:%d h:%d\n", x, y, width, height);
+  g_debug ("gl window props: x:%d y:%d\n", x, y);
 
   wm_atoms[0] = XInternAtom (priv->device, "WM_DELETE_WINDOW", True);
   if (wm_atoms[0] == None)
@@ -380,6 +390,11 @@ gst_gl_window_new (gint width, gint height, gulong external_gl_context)
       glXCreateContext (priv->device, priv->visual_info,
       (GLXContext) external_gl_context, TRUE);
 
+  if (!priv->gl_context) {
+    g_debug ("failed to create opengl context\n");
+    goto failure;
+  }
+
   g_debug ("gl context id: %ld\n", (gulong) priv->gl_context);
 
   if (!glXIsDirect (priv->device, priv->gl_context))
@@ -398,8 +413,10 @@ gst_gl_window_new (gint width, gint height, gulong external_gl_context)
 
   ret = glXMakeCurrent (priv->device, priv->internal_win_id, priv->gl_context);
 
-  if (!ret)
+  if (!ret) {
     g_debug ("failed to make opengl context current\n");
+    goto failure;
+  }
 
   if (glXIsDirect (priv->device, priv->gl_context))
     g_debug ("Direct Rendering: yes\n");
@@ -409,9 +426,10 @@ gst_gl_window_new (gint width, gint height, gulong external_gl_context)
   g_mutex_unlock (priv->x_lock);
 
   return window;
-no_display:
+
+failure:
   g_mutex_unlock (priv->x_lock);
-  g_object_unref (window);
+  g_object_unref (G_OBJECT (window));
   return NULL;
 }
 
@@ -421,6 +439,41 @@ gst_gl_window_error_quark (void)
   return g_quark_from_static_string ("gst-gl-window-error");
 }
 
+gulong
+gst_gl_window_get_internal_gl_context (GstGLWindow * window)
+{
+  GstGLWindowPrivate *priv = window->priv;
+  return (gulong) priv->gl_context;
+}
+
+void
+callback_activate_gl_context (GstGLWindowPrivate * priv)
+{
+  if (!glXMakeCurrent (priv->device, priv->internal_win_id, priv->gl_context))
+    g_debug ("failed to activate opengl context %lud\n",
+        (gulong) priv->gl_context);
+}
+
+void
+callback_inactivate_gl_context (GstGLWindowPrivate * priv)
+{
+  if (!glXMakeCurrent (priv->device, None, NULL))
+    g_debug ("failed to inactivate opengl context %lud\n",
+        (gulong) priv->gl_context);
+}
+
+void
+gst_gl_window_activate_gl_context (GstGLWindow * window, gboolean activate)
+{
+  GstGLWindowPrivate *priv = window->priv;
+  if (activate)
+    gst_gl_window_send_message (window,
+        GST_GL_WINDOW_CB (callback_activate_gl_context), priv);
+  else
+    gst_gl_window_send_message (window,
+        GST_GL_WINDOW_CB (callback_inactivate_gl_context), priv);
+}
+
 /* Not called by the gl thread */
 void
 gst_gl_window_set_external_window_id (GstGLWindow * window, gulong id)
@@ -493,7 +546,7 @@ gst_gl_window_set_close_callback (GstGLWindow * window, GstGLWindowCB callback,
 
 /* Called in the gl thread */
 void
-gst_gl_window_draw_unlocked (GstGLWindow * window)
+gst_gl_window_draw_unlocked (GstGLWindow * window, gint width, gint height)
 {
   GstGLWindowPrivate *priv = window->priv;
 
@@ -521,7 +574,7 @@ gst_gl_window_draw_unlocked (GstGLWindow * window)
 
 /* Not called by the gl thread */
 void
-gst_gl_window_draw (GstGLWindow * window)
+gst_gl_window_draw (GstGLWindow * window, gint width, gint height)
 {
   if (window) {
     GstGLWindowPrivate *priv = window->priv;
@@ -532,13 +585,22 @@ gst_gl_window_draw (GstGLWindow * window)
       XEvent event;
       XWindowAttributes attr;
 
+      XGetWindowAttributes (priv->disp_send, priv->internal_win_id, &attr);
+
       if (!priv->visible) {
+
+        if (!priv->parent) {
+          attr.width = width;
+          attr.height = height;
+          XResizeWindow (priv->disp_send, priv->internal_win_id,
+              attr.width, attr.height);
+          XSync (priv->disp_send, FALSE);
+        }
+
         XMapWindow (priv->disp_send, priv->internal_win_id);
         priv->visible = TRUE;
       }
 
-      XGetWindowAttributes (priv->disp_send, priv->internal_win_id, &attr);
-
       if (priv->parent) {
         XWindowAttributes attr_parent;
         XGetWindowAttributes (priv->disp_send, priv->parent, &attr_parent);
@@ -598,7 +660,7 @@ gst_gl_window_run_loop (GstGLWindow * window)
     g_mutex_lock (priv->x_lock);
 
     // use in generic/cube and other related uses
-    priv->allow_extra_expose_events = XPending (priv->device) <= 2;
+    priv->allow_extra_expose_events = XPending (priv->device) <= 0;
 
     switch (event.type) {
       case ClientMessage:
@@ -762,7 +824,7 @@ gst_gl_window_run_loop (GstGLWindow * window)
       }
 
       default:
-        g_debug ("unknow\n");
+        g_debug ("unknown XEvent type: %ud\n", event.type);
         break;
 
     }                           // switch
