@@ -20,10 +20,8 @@
 
 #include <gstgleffects.h>
 
-static gfloat gauss_kernel[9] = { 0.060493f, 0.075284f, 0.088016f,
-  0.096667f, 0.099736f, 0.096667f,
-  0.088016f, 0.075284f, 0.060493f
-};
+static gboolean kernel_ready = FALSE;
+static float gauss_kernel[7];
 
 static void
 gst_gl_effects_glow_step_one (gint width, gint height, guint texture,
@@ -40,8 +38,14 @@ gst_gl_effects_glow_step_one (gint width, gint height, guint texture,
     g_hash_table_insert (effects->shaderstable, "glow0", shader);
   }
 
-  g_return_if_fail (gst_gl_shader_compile_and_check (shader,
-          luma_threshold_fragment_source, GST_GL_SHADER_FRAGMENT_SOURCE));
+  if (!gst_gl_shader_compile_and_check (shader,
+          luma_threshold_fragment_source, GST_GL_SHADER_FRAGMENT_SOURCE)) {
+    gst_gl_display_set_error (GST_GL_FILTER (effects)->display,
+        "Failed to initialize luma threshold shader");
+    GST_ELEMENT_ERROR (effects, RESOURCE, NOT_FOUND,
+        GST_GL_DISPLAY_ERR_MSG (GST_GL_FILTER (effects)->display), (NULL));
+    return;
+  }
 
   glMatrixMode (GL_PROJECTION);
   glLoadIdentity ();
@@ -64,9 +68,6 @@ gst_gl_effects_glow_step_two (gint width, gint height, guint texture,
   GstGLEffects *effects = GST_GL_EFFECTS (stuff);
   GstGLShader *shader;
 
-  /* hard coded kernel, it could be easily generated at runtime with a
-   * property to change standard deviation */
-
   shader = g_hash_table_lookup (effects->shaderstable, "glow1");
 
   if (!shader) {
@@ -74,8 +75,19 @@ gst_gl_effects_glow_step_two (gint width, gint height, guint texture,
     g_hash_table_insert (effects->shaderstable, "glow1", shader);
   }
 
-  g_return_if_fail (gst_gl_shader_compile_and_check (shader,
-          hconv9_fragment_source, GST_GL_SHADER_FRAGMENT_SOURCE));
+  if (!kernel_ready) {
+    fill_gaussian_kernel (gauss_kernel, 7, 10.0);
+    kernel_ready = TRUE;
+  }
+
+  if (!gst_gl_shader_compile_and_check (shader,
+          hconv7_fragment_source, GST_GL_SHADER_FRAGMENT_SOURCE)) {
+    gst_gl_display_set_error (GST_GL_FILTER (effects)->display,
+        "Failed to initialize hconv7 shader");
+    GST_ELEMENT_ERROR (effects, RESOURCE, NOT_FOUND,
+        GST_GL_DISPLAY_ERR_MSG (GST_GL_FILTER (effects)->display), (NULL));
+    return;
+  }
 
   glMatrixMode (GL_PROJECTION);
   glLoadIdentity ();
@@ -88,10 +100,7 @@ gst_gl_effects_glow_step_two (gint width, gint height, guint texture,
   glDisable (GL_TEXTURE_RECTANGLE_ARB);
 
   gst_gl_shader_set_uniform_1i (shader, "tex", 1);
-
-  gst_gl_shader_set_uniform_1fv (shader, "kernel", 9, gauss_kernel);
-  gst_gl_shader_set_uniform_1f (shader, "norm_const", 0.740656f);
-  gst_gl_shader_set_uniform_1f (shader, "norm_offset", 0.0f);
+  gst_gl_shader_set_uniform_1fv (shader, "kernel", 7, gauss_kernel);
 
   gst_gl_effects_draw_texture (effects, texture);
 }
@@ -110,8 +119,14 @@ gst_gl_effects_glow_step_three (gint width, gint height, guint texture,
     g_hash_table_insert (effects->shaderstable, "glow2", shader);
   }
 
-  g_return_if_fail (gst_gl_shader_compile_and_check (shader,
-          vconv9_fragment_source, GST_GL_SHADER_FRAGMENT_SOURCE));
+  if (!gst_gl_shader_compile_and_check (shader,
+          vconv7_fragment_source, GST_GL_SHADER_FRAGMENT_SOURCE)) {
+    gst_gl_display_set_error (GST_GL_FILTER (effects)->display,
+        "Failed to initialize vcon7 shader");
+    GST_ELEMENT_ERROR (effects, RESOURCE, NOT_FOUND,
+        GST_GL_DISPLAY_ERR_MSG (GST_GL_FILTER (effects)->display), (NULL));
+    return;
+  }
 
   glMatrixMode (GL_PROJECTION);
   glLoadIdentity ();
@@ -124,10 +139,7 @@ gst_gl_effects_glow_step_three (gint width, gint height, guint texture,
   glDisable (GL_TEXTURE_RECTANGLE_ARB);
 
   gst_gl_shader_set_uniform_1i (shader, "tex", 1);
-
-  gst_gl_shader_set_uniform_1fv (shader, "kernel", 9, gauss_kernel);
-  gst_gl_shader_set_uniform_1f (shader, "norm_const", 0.740656f);
-  gst_gl_shader_set_uniform_1f (shader, "norm_offset", 0.0f);
+  gst_gl_shader_set_uniform_1fv (shader, "kernel", 7, gauss_kernel);
 
   gst_gl_effects_draw_texture (effects, texture);
 }
@@ -146,8 +158,14 @@ gst_gl_effects_glow_step_four (gint width, gint height, guint texture,
     g_hash_table_insert (effects->shaderstable, "glow3", shader);
   }
 
-  g_return_if_fail (gst_gl_shader_compile_and_check (shader,
-          sum_fragment_source, GST_GL_SHADER_FRAGMENT_SOURCE));
+  if (!gst_gl_shader_compile_and_check (shader,
+          sum_fragment_source, GST_GL_SHADER_FRAGMENT_SOURCE)) {
+    gst_gl_display_set_error (GST_GL_FILTER (effects)->display,
+        "Failed to initialize sum shader");
+    GST_ELEMENT_ERROR (effects, RESOURCE, NOT_FOUND,
+        GST_GL_DISPLAY_ERR_MSG (GST_GL_FILTER (effects)->display), (NULL));
+    return;
+  }
 
   glMatrixMode (GL_PROJECTION);
   glLoadIdentity ();
