@@ -41,15 +41,14 @@
 #include <png.h>
 #include "gstglbumper.h"
 
+#if PNG_LIBPNG_VER >= 10400
+#define int_p_NULL         NULL
+#define png_infopp_NULL    NULL
+#endif
+
 #define GST_CAT_DEFAULT gst_gl_bumper_debug
 GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
 
-static const GstElementDetails element_details =
-GST_ELEMENT_DETAILS ("OpenGL bumper filter",
-    "Filter/Effect",
-    "Bump mapping filter",
-    "Cyril Comparon <cyril.comparon@gmail.com>, Julien Isorce <julien.isorce@gmail.com>");
-
 enum
 {
   PROP_0,
@@ -68,7 +67,7 @@ static void gst_gl_bumper_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
 static void gst_gl_bumper_reset (GstGLFilter * filter);
-static void gst_gl_bumper_init_shader (GstGLFilter * filter);
+static gboolean gst_gl_bumper_init_shader (GstGLFilter * filter);
 static gboolean gst_gl_bumper_filter (GstGLFilter * filter,
     GstGLBuffer * inbuf, GstGLBuffer * outbuf);
 static void gst_gl_bumper_callback (gint width, gint height, guint texture,
@@ -168,6 +167,8 @@ gst_gl_bumper_init_resources (GstGLFilter * filter)
   guint y = 0;
   guchar *raw_data = NULL;
   guchar **rows = NULL;
+  png_byte magic[8];
+  gint n_read;
 
   if (!filter->display)
     return;
@@ -177,6 +178,19 @@ gst_gl_bumper_init_resources (GstGLFilter * filter)
   if ((fp = fopen (bumper->location, "rb")) == NULL)
     LOAD_ERROR ("file not found");
 
+  /* Read magic number */
+  n_read = fread (magic, 1, sizeof (magic), fp);
+  if (n_read != sizeof (magic)) {
+    fclose (fp);
+    LOAD_ERROR ("can't read PNG magic number");
+  }
+
+  /* Check for valid magic number */
+  if (png_sig_cmp (magic, 0, sizeof (magic))) {
+    fclose (fp);
+    LOAD_ERROR ("not a valid PNG image");
+  }
+
   png_ptr = png_create_read_struct (PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
 
   if (png_ptr == NULL) {
@@ -254,7 +268,10 @@ gst_gl_bumper_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_set_details (element_class, &element_details);
+  gst_element_class_set_details_simple (element_class, "OpenGL bumper filter",
+      "Filter/Effect", "Bump mapping filter",
+      "Cyril Comparon <cyril.comparon@gmail.com>, "
+      "Julien Isorce <julien.isorce@gmail.com>");
 }
 
 static void
@@ -275,7 +292,8 @@ gst_gl_bumper_class_init (GstGLBumperClass * klass)
   g_object_class_install_property (gobject_class,
       PROP_LOCATION, g_param_spec_string ("location",
           "Normal map location",
-          "Normal map location", NULL, G_PARAM_READWRITE));
+          "Normal map location", NULL,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -331,13 +349,13 @@ gst_gl_bumper_get_property (GObject * object, guint prop_id,
   }
 }
 
-static void
+static gboolean
 gst_gl_bumper_init_shader (GstGLFilter * filter)
 {
   GstGLBumper *bumper = GST_GL_BUMPER (filter);
 
   //blocking call, wait the opengl thread has compiled the shader
-  gst_gl_display_gen_shader (filter->display, bumper_v_src, bumper_f_src,
+  return gst_gl_display_gen_shader (filter->display, bumper_v_src, bumper_f_src,
       &bumper->shader);
 }
 
