@@ -27,7 +27,7 @@
  * <refsect2>
  * <title>Color space conversion</title>
  * <para>
- * When needed, the color space conversion is made in a fragment shader using 
+ * When needed, the color space conversion is made in a fragment shader using
  * one frame buffer object instance.
  * </para>
  * </refsect2>
@@ -48,14 +48,14 @@
  * |[
  * gst-launch -v gltestsrc ! gldownload ! xvimagesink
  * ]| A pipeline to test hardware colorspace conversion.
- * Your driver must support GLSL (OpenGL Shading Language needs OpenGL >= 2.1). 
+ * Your driver must support GLSL (OpenGL Shading Language needs OpenGL >= 2.1).
  * Texture RGB32 is converted to one of the 4 following format YUY2, UYVY, I420, YV12 and AYUV,
  * through some fragment shaders and using one framebuffer (FBO extension OpenGL >= 1.4).
  * MESA >= 7.1 supports GLSL but it's made in software.
  * |[
  * gst-launch -v videotestsrc ! glupload ! gldownload ! "video/x-raw-yuv, format=(fourcc)YUY2" ! glimagesink
  * ]| A pipeline to test hardware colorspace conversion
- * FBO and GLSL are required. 
+ * FBO and GLSL are required.
  * </refsect2>
  */
 
@@ -68,15 +68,9 @@
 #define GST_CAT_DEFAULT gst_gl_download_debug
 GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
 
-static const GstElementDetails element_details =
-GST_ELEMENT_DETAILS ("OpenGL video maker",
-    "Filter/Effect",
-    "A from GL to video flow filter",
-    "Julien Isorce <julien.isorce@gmail.com>");
-
 #ifndef OPENGL_ES2
 #define ADDITIONAL_CAPS \
-        
+
 #else
 #define ADDITIONAL_CAPS
 #endif
@@ -86,11 +80,10 @@ static GstStaticPadTemplate gst_gl_download_src_pad_template =
     GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (
-        GST_VIDEO_CAPS_RGB  ";"
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_RGB ";"
         GST_VIDEO_CAPS_RGBx ";"
         GST_VIDEO_CAPS_RGBA ";"
-        GST_VIDEO_CAPS_BGR  ";"
+        GST_VIDEO_CAPS_BGR ";"
         GST_VIDEO_CAPS_BGRx ";"
         GST_VIDEO_CAPS_BGRA ";"
         GST_VIDEO_CAPS_xRGB ";"
@@ -104,11 +97,9 @@ static GstStaticPadTemplate gst_gl_download_src_pad_template =
     GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (
-        GST_VIDEO_CAPS_RGB  ";"
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_RGB ";"
         GST_VIDEO_CAPS_RGBx ";"
-        GST_VIDEO_CAPS_RGBA ";"
-        GST_VIDEO_CAPS_YUV ("{ YUY2, UYVY, AYUV }"))
+        GST_VIDEO_CAPS_RGBA ";" GST_VIDEO_CAPS_YUV ("{ YUY2, UYVY, AYUV }"))
     );
 #endif
 
@@ -135,6 +126,8 @@ static void gst_gl_download_set_property (GObject * object, guint prop_id,
 static void gst_gl_download_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
+static gboolean gst_gl_download_src_query (GstPad * pad, GstQuery * query);
+
 static void gst_gl_download_reset (GstGLDownload * download);
 static gboolean gst_gl_download_set_caps (GstBaseTransform * bt,
     GstCaps * incaps, GstCaps * outcaps);
@@ -153,7 +146,9 @@ gst_gl_download_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_set_details (element_class, &element_details);
+  gst_element_class_set_details_simple (element_class, "OpenGL video maker",
+      "Filter/Effect", "A from GL to video flow filter",
+      "Julien Isorce <julien.isorce@gmail.com>");
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_gl_download_src_pad_template));
@@ -185,6 +180,11 @@ gst_gl_download_class_init (GstGLDownloadClass * klass)
 static void
 gst_gl_download_init (GstGLDownload * download, GstGLDownloadClass * klass)
 {
+  GstBaseTransform *base_trans = GST_BASE_TRANSFORM (download);
+
+  gst_pad_set_query_function (base_trans->srcpad,
+      GST_DEBUG_FUNCPTR (gst_gl_download_src_query));
+
   gst_gl_download_reset (download);
 }
 
@@ -216,6 +216,30 @@ gst_gl_download_get_property (GObject * object, guint prop_id,
   }
 }
 
+static gboolean
+gst_gl_download_src_query (GstPad * pad, GstQuery * query)
+{
+  GstGLDownload *download = GST_GL_DOWNLOAD (gst_pad_get_parent (pad));
+  gboolean res = FALSE;
+
+  switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_CUSTOM:
+    {
+      GstStructure *structure = gst_query_get_structure (query);
+      gst_structure_set (structure, "gstgldisplay", G_TYPE_POINTER,
+          download->display, NULL);
+      res = gst_pad_query_default (pad, query);
+      break;
+    }
+    default:
+      res = gst_pad_query_default (pad, query);
+      break;
+  }
+
+  gst_object_unref (download);
+
+  return res;
+}
 
 static void
 gst_gl_download_reset (GstGLDownload * download)
@@ -230,7 +254,14 @@ gst_gl_download_reset (GstGLDownload * download)
 static gboolean
 gst_gl_download_start (GstBaseTransform * bt)
 {
-  //GstGLDownload* download = GST_GL_DOWNLOAD (bt);
+  GstGLDownload *download = GST_GL_DOWNLOAD (bt);
+
+  download->display = gst_gl_display_new ();
+  if (!gst_gl_display_create_context (download->display, 0)) {
+    GST_ELEMENT_ERROR (download, RESOURCE, NOT_FOUND,
+        GST_GL_DISPLAY_ERR_MSG (download->display), (NULL));
+    return FALSE;
+  }
 
   return TRUE;
 }
@@ -249,7 +280,6 @@ static GstCaps *
 gst_gl_download_transform_caps (GstBaseTransform * bt,
     GstPadDirection direction, GstCaps * caps)
 {
-  GstGLDownload *download;
   GstStructure *structure;
   GstCaps *newcaps, *newothercaps;
   GstStructure *newstruct;
@@ -258,8 +288,6 @@ gst_gl_download_transform_caps (GstBaseTransform * bt,
   const GValue *framerate_value;
   const GValue *par_value;
 
-  download = GST_GL_DOWNLOAD (bt);
-
   GST_DEBUG ("transform caps %" GST_PTR_FORMAT, caps);
 
   structure = gst_caps_get_structure (caps, 0);
@@ -319,6 +347,17 @@ gst_gl_download_set_caps (GstBaseTransform * bt, GstCaps * incaps,
     return FALSE;
   }
 
+  if (!download->display) {
+    GST_ERROR ("display is null");
+    return FALSE;
+  }
+  //blocking call, init color space conversion if needed
+  ret = gst_gl_display_init_download (download->display, download->video_format,
+      download->width, download->height);
+  if (!ret)
+    GST_ELEMENT_ERROR (download, RESOURCE, NOT_FOUND,
+        GST_GL_DISPLAY_ERR_MSG (download->display), (NULL));
+
   return ret;
 }
 
@@ -353,15 +392,6 @@ gst_gl_download_transform (GstBaseTransform * trans, GstBuffer * inbuf,
   GstGLDownload *download = GST_GL_DOWNLOAD (trans);
   GstGLBuffer *gl_inbuf = GST_GL_BUFFER (inbuf);
 
-  if (download->display == NULL) {
-    download->display = g_object_ref (gl_inbuf->display);
-
-    //blocking call, init color space conversion if needed
-    gst_gl_display_init_download (download->display, download->video_format,
-        download->width, download->height);
-  } else
-    g_assert (download->display == gl_inbuf->display);
-
   GST_DEBUG ("making video %p size %d",
       GST_BUFFER_DATA (outbuf), GST_BUFFER_SIZE (outbuf));
 
