@@ -45,17 +45,12 @@
 #define GST_CAT_DEFAULT gst_gl_filter_app_debug
 GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
 
-static const GstElementDetails element_details =
-GST_ELEMENT_DETAILS ("OpenGL application filter",
-    "Filter/Effect",
-    "Use client callbacks to define the scene",
-    "Julien Isorce <julien.isorce@gmail.com>");
-
 enum
 {
   PROP_0,
   PROP_CLIENT_RESHAPE_CALLBACK,
-  PROP_CLIENT_DRAW_CALLBACK
+  PROP_CLIENT_DRAW_CALLBACK,
+  PROP_CLIENT_DATA
 };
 
 #define DEBUG_INIT(bla) \
@@ -82,7 +77,10 @@ gst_gl_filter_app_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_set_details (element_class, &element_details);
+  gst_element_class_set_details_simple (element_class,
+      "OpenGL application filter", "Filter/Effect",
+      "Use client callbacks to define the scene",
+      "Julien Isorce <julien.isorce@gmail.com>");
 }
 
 static void
@@ -98,14 +96,20 @@ gst_gl_filter_app_class_init (GstGLFilterAppClass * klass)
   GST_GL_FILTER_CLASS (klass)->filter = gst_gl_filter_app_filter;
 
   g_object_class_install_property (gobject_class, PROP_CLIENT_RESHAPE_CALLBACK,
-      g_param_spec_pointer ("client_reshape_callback",
+      g_param_spec_pointer ("client-reshape-callback",
           "Client reshape callback",
           "Define a custom reshape callback in a client code",
-          G_PARAM_WRITABLE));
+          G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_CLIENT_DRAW_CALLBACK,
-      g_param_spec_pointer ("client_draw_callback", "Client draw callback",
-          "Define a custom draw callback in a client code", G_PARAM_WRITABLE));
+      g_param_spec_pointer ("client-draw-callback", "Client draw callback",
+          "Define a custom draw callback in a client code",
+          G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property (gobject_class, PROP_CLIENT_DATA,
+      g_param_spec_pointer ("client-data", "Client data",
+          "Pass data to the draw and reshape callbacks",
+          G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -113,6 +117,7 @@ gst_gl_filter_app_init (GstGLFilterApp * filter, GstGLFilterAppClass * klass)
 {
   filter->clientReshapeCallback = NULL;
   filter->clientDrawCallback = NULL;
+  filter->client_data = NULL;
 }
 
 static void
@@ -132,6 +137,11 @@ gst_gl_filter_app_set_property (GObject * object, guint prop_id,
       filter->clientDrawCallback = g_value_get_pointer (value);
       break;
     }
+    case PROP_CLIENT_DATA:
+    {
+      filter->client_data = g_value_get_pointer (value);
+      break;
+    }
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -172,7 +182,8 @@ gst_gl_filter_app_filter (GstGLFilter * filter, GstGLBuffer * inbuf,
         filter->fbo, filter->depthbuffer, outbuf->texture,
         app_filter->clientDrawCallback, inbuf->width, inbuf->height,
         inbuf->texture, 45, (gfloat) filter->width / (gfloat) filter->height,
-        0.1, 100, GST_GL_DISPLAY_PROJECTION_PERSPECTIVE, NULL);
+        0.1, 100, GST_GL_DISPLAY_PROJECTION_PERSPECTIVE,
+        app_filter->client_data);
   }
   //default
   else {
