@@ -1,4 +1,4 @@
-/* 
+/*
  * GStreamer
  * Copyright (C) 2008 Julien Isorce <julien.isorce@gmail.com>
  *
@@ -49,12 +49,6 @@
 #define GST_CAT_DEFAULT gst_gl_filter_cube_debug
 GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
 
-static const GstElementDetails element_details =
-GST_ELEMENT_DETAILS ("OpenGL cube filter",
-    "Filter/Effect",
-    "Map input texture on the 6 cube faces",
-    "Julien Isorce <julien.isorce@gmail.com>");
-
 enum
 {
   PROP_0,
@@ -82,7 +76,7 @@ static gboolean gst_gl_filter_cube_set_caps (GstGLFilter * filter,
     GstCaps * incaps, GstCaps * outcaps);
 #ifdef OPENGL_ES2
 static void gst_gl_filter_cube_reset (GstGLFilter * filter);
-static void gst_gl_filter_cube_init_shader (GstGLFilter * filter);
+static gboolean gst_gl_filter_cube_init_shader (GstGLFilter * filter);
 #endif
 static gboolean gst_gl_filter_cube_filter (GstGLFilter * filter,
     GstGLBuffer * inbuf, GstGLBuffer * outbuf);
@@ -118,7 +112,7 @@ static const gchar *cube_v_src =
     "     -sin(zrot),  cos(zrot),        0.0, 0.0,                \n"
     "            0.0,        0.0,        1.0, 0.0,                \n"
     "            0.0,        0.0,        0.0, 1.0 );              \n"
-    "   gl_Position = matZ * matY * matX * u_matrix * a_position; \n"
+    "   gl_Position = u_matrix * matZ * matY * matX * a_position; \n"
     "   v_texCoord = a_texCoord;                                  \n"
     "}                                                            \n";
 
@@ -138,7 +132,9 @@ gst_gl_filter_cube_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_set_details (element_class, &element_details);
+  gst_element_class_set_details_simple (element_class, "OpenGL cube filter",
+      "Filter/Effect", "Map input texture on the 6 cube faces",
+      "Julien Isorce <julien.isorce@gmail.com>");
 }
 
 static void
@@ -159,33 +155,34 @@ gst_gl_filter_cube_class_init (GstGLFilterCubeClass * klass)
 
   g_object_class_install_property (gobject_class, PROP_RED,
       g_param_spec_float ("red", "Red", "Background red color",
-          0.0f, 1.0f, 0.0f, G_PARAM_WRITABLE));
+          0.0f, 1.0f, 0.0f, G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_GREEN,
       g_param_spec_float ("green", "Green", "Background reen color",
-          0.0f, 1.0f, 0.0f, G_PARAM_WRITABLE));
+          0.0f, 1.0f, 0.0f, G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_BLUE,
       g_param_spec_float ("blue", "Blue", "Background blue color",
-          0.0f, 1.0f, 0.0f, G_PARAM_WRITABLE));
+          0.0f, 1.0f, 0.0f, G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_FOVY,
       g_param_spec_double ("fovy", "Fovy", "Field of view angle in degrees",
-          0.0, 180.0, 45.0, G_PARAM_WRITABLE));
+          0.0, 180.0, 45.0, G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_ASPECT,
       g_param_spec_double ("aspect", "Aspect",
-          "Field of view in the x direction", 0.0, 100, 0.0, G_PARAM_WRITABLE));
+          "Field of view in the x direction", 0.0, 100, 0.0,
+          G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_ZNEAR,
       g_param_spec_double ("znear", "Znear",
           "Specifies the distance from the viewer to the near clipping plane",
-          0.0, 100.0, 0.1, G_PARAM_WRITABLE));
+          0.0, 100.0, 0.1, G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_ZFAR,
       g_param_spec_double ("zfar", "Zfar",
           "Specifies the distance from the viewer to the far clipping plane",
-          0.0, 1000.0, 100.0, G_PARAM_WRITABLE));
+          0.0, 1000.0, 100.0, G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -269,13 +266,13 @@ gst_gl_filter_cube_reset (GstGLFilter * filter)
   gst_gl_display_del_shader (filter->display, cube_filter->shader);
 }
 
-static void
+static gboolean
 gst_gl_filter_cube_init_shader (GstGLFilter * filter)
 {
   GstGLFilterCube *cube_filter = GST_GL_FILTER_CUBE (filter);
 
   //blocking call, wait the opengl thread has compiled the shader
-  gst_gl_display_gen_shader (filter->display, cube_v_src, cube_f_src,
+  return gst_gl_display_gen_shader (filter->display, cube_v_src, cube_f_src,
       &cube_filter->shader);
 }
 #endif
@@ -319,7 +316,9 @@ gst_gl_filter_cube_callback (gint width, gint height, guint texture,
       GL_CLAMP_TO_EDGE);
   glTexParameteri (GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T,
       GL_CLAMP_TO_EDGE);
+#ifndef OPENGL_ES2
   glTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
+#endif
 
   glClearColor (cube_filter->red, cube_filter->green, cube_filter->blue, 0.0);
   glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
