@@ -46,12 +46,6 @@
 #define GST_CAT_DEFAULT gst_gl_filter_glass_debug
 GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
 
-static const GstElementDetails element_details =
-GST_ELEMENT_DETAILS ("OpenGL glass filter",
-    "Filter/Effect",
-    "Glass Filter",
-    "Julien Isorce <julien.isorce@gmail.com>");
-
 enum
 {
   PROP_0
@@ -69,7 +63,7 @@ static void gst_gl_filter_glass_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
 static void gst_gl_filter_glass_reset (GstGLFilter * filter);
-static void gst_gl_filter_glass_init_shader (GstGLFilter * filter);
+static gboolean gst_gl_filter_glass_init_shader (GstGLFilter * filter);
 static gboolean gst_gl_filter_glass_filter (GstGLFilter * filter,
     GstGLBuffer * inbuf, GstGLBuffer * outbuf);
 
@@ -110,7 +104,9 @@ gst_gl_filter_glass_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_set_details (element_class, &element_details);
+  gst_element_class_set_details_simple (element_class, "OpenGL glass filter",
+      "Filter/Effect", "Glass Filter",
+      "Julien Isorce <julien.isorce@gmail.com>");
 }
 
 static void
@@ -170,13 +166,13 @@ gst_gl_filter_glass_get_property (GObject * object, guint prop_id,
   }
 }
 
-static void
+static gboolean
 gst_gl_filter_glass_init_shader (GstGLFilter * filter)
 {
   GstGLFilterGlass *glass_filter = GST_GL_FILTER_GLASS (filter);
 
   //blocking call, wait the opengl thread has compiled the shader
-  gst_gl_display_gen_shader (filter->display, 0, glass_fragment_source,
+  return gst_gl_display_gen_shader (filter->display, 0, glass_fragment_source,
       &glass_filter->shader);
 }
 
