@@ -36,7 +36,7 @@
  * Depends on the driver, OpenGL handles hardware accelerated
  * scaling of video frames. This means that the element will just accept
  * incoming video frames no matter their geometry and will then put them to the
- * drawable scaling them on the fly. Using the #GstXvImageSink:force-aspect-ratio
+ * drawable scaling them on the fly. Using the #GstGLImageSink:force-aspect-ratio
  * property it is possible to enforce scaling with a constant aspect ratio,
  * which means drawing black borders around the video frame.
  * </para>
@@ -98,6 +98,8 @@ static void gst_glimage_sink_set_property (GObject * object, guint prop_id,
 static void gst_glimage_sink_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * param_spec);
 
+static gboolean gst_glimage_sink_query (GstElement * element, GstQuery * query);
+
 static GstStateChangeReturn
 gst_glimage_sink_change_state (GstElement * element, GstStateChange transition);
 
@@ -106,24 +108,16 @@ static void gst_glimage_sink_get_times (GstBaseSink * bsink, GstBuffer * buf,
 static gboolean gst_glimage_sink_set_caps (GstBaseSink * bsink, GstCaps * caps);
 static GstFlowReturn gst_glimage_sink_render (GstBaseSink * bsink,
     GstBuffer * buf);
-static gboolean gst_glimage_sink_start (GstBaseSink * bsink);
-static gboolean gst_glimage_sink_stop (GstBaseSink * bsink);
 
 static void gst_glimage_sink_xoverlay_init (GstXOverlayClass * iface);
-static void gst_glimage_sink_set_xwindow_id (GstXOverlay * overlay,
-    gulong window_id);
+static void gst_glimage_sink_set_window_handle (GstXOverlay * overlay,
+    guintptr id);
 static void gst_glimage_sink_expose (GstXOverlay * overlay);
 static gboolean gst_glimage_sink_interface_supported (GstImplementsInterface *
     iface, GType type);
 static void gst_glimage_sink_implements_init (GstImplementsInterfaceClass *
     klass);
 
-static const GstElementDetails gst_glimage_sink_details =
-GST_ELEMENT_DETAILS ("OpenGL video sink",
-    "Sink/Video",
-    "A videosink based on OpenGL",
-    "Julien Isorce <julien.isorce@gmail.com>");
-
 #ifndef OPENGL_ES2
 static GstStaticPadTemplate gst_glimage_sink_template =
     GST_STATIC_PAD_TEMPLATE ("sink",
@@ -142,9 +136,8 @@ static GstStaticPadTemplate gst_glimage_sink_template =
     GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (
-        GST_GL_VIDEO_CAPS ";"
-        GST_VIDEO_CAPS_RGB  ";" 
+    GST_STATIC_CAPS (GST_GL_VIDEO_CAPS ";"
+        GST_VIDEO_CAPS_RGB ";"
         GST_VIDEO_CAPS_RGBx ";"
         GST_VIDEO_CAPS_RGBA ";"
         GST_VIDEO_CAPS_YUV ("{ I420, YV12, YUY2, UYVY, AYUV }"))
@@ -157,7 +150,9 @@ enum
   ARG_DISPLAY,
   PROP_CLIENT_RESHAPE_CALLBACK,
   PROP_CLIENT_DRAW_CALLBACK,
-  PROP_FORCE_ASPECT_RATIO
+  PROP_CLIENT_DATA,
+  PROP_FORCE_ASPECT_RATIO,
+  PROP_PIXEL_ASPECT_RATIO
 };
 
 GST_BOILERPLATE_FULL (GstGLImageSink, gst_glimage_sink, GstVideoSink,
@@ -193,10 +188,12 @@ gst_glimage_sink_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_details (element_class, &gst_glimage_sink_details);
+  gst_element_class_set_details_simple (element_class, "OpenGL video sink",
+      "Sink/Video", "A videosink based on OpenGL",
+      "Julien Isorce <julien.isorce@gmail.com>");
+
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_glimage_sink_template));
-
 }
 
 static void
@@ -215,34 +212,44 @@ gst_glimage_sink_class_init (GstGLImageSinkClass * klass)
 
   g_object_class_install_property (gobject_class, ARG_DISPLAY,
       g_param_spec_string ("display", "Display", "Display name",
-          NULL, G_PARAM_READWRITE));
+          NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_CLIENT_RESHAPE_CALLBACK,
-      g_param_spec_pointer ("client_reshape_callback",
+      g_param_spec_pointer ("client-reshape-callback",
           "Client reshape callback",
           "Define a custom reshape callback in a client code",
-          G_PARAM_WRITABLE));
+          G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_CLIENT_DRAW_CALLBACK,
-      g_param_spec_pointer ("client_draw_callback", "Client draw callback",
-          "Define a custom draw callback in a client code", G_PARAM_WRITABLE));
+      g_param_spec_pointer ("client-draw-callback", "Client draw callback",
+          "Define a custom draw callback in a client code",
+          G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property (gobject_class, PROP_CLIENT_DATA,
+      g_param_spec_pointer ("client-data", "Client data",
+          "Pass data to the draw and reshape callbacks",
+          G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_FORCE_ASPECT_RATIO,
       g_param_spec_boolean ("force-aspect-ratio",
           "Force aspect ratio",
           "When enabled, scaling will respect original aspect ratio", FALSE,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property (gobject_class, PROP_PIXEL_ASPECT_RATIO,
+      g_param_spec_string ("pixel-aspect-ratio", "Pixel Aspect Ratio",
+          "The pixel aspect ratio of the device", "1/1",
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gobject_class->finalize = gst_glimage_sink_finalize;
 
   gstelement_class->change_state = gst_glimage_sink_change_state;
+  gstelement_class->query = GST_DEBUG_FUNCPTR (gst_glimage_sink_query);
 
   gstbasesink_class->set_caps = gst_glimage_sink_set_caps;
   gstbasesink_class->get_times = gst_glimage_sink_get_times;
   gstbasesink_class->preroll = gst_glimage_sink_render;
   gstbasesink_class->render = gst_glimage_sink_render;
-  gstbasesink_class->start = gst_glimage_sink_start;
-  gstbasesink_class->stop = gst_glimage_sink_stop;
 }
 
 static void
@@ -256,7 +263,9 @@ gst_glimage_sink_init (GstGLImageSink * glimage_sink,
   glimage_sink->stored_buffer = NULL;
   glimage_sink->clientReshapeCallback = NULL;
   glimage_sink->clientDrawCallback = NULL;
+  glimage_sink->client_data = NULL;
   glimage_sink->keep_aspect_ratio = FALSE;
+  glimage_sink->par = NULL;
 }
 
 static void
@@ -286,11 +295,27 @@ gst_glimage_sink_set_property (GObject * object, guint prop_id,
       glimage_sink->clientDrawCallback = g_value_get_pointer (value);
       break;
     }
+    case PROP_CLIENT_DATA:
+    {
+      glimage_sink->client_data = g_value_get_pointer (value);
+      break;
+    }
     case PROP_FORCE_ASPECT_RATIO:
     {
       glimage_sink->keep_aspect_ratio = g_value_get_boolean (value);
       break;
     }
+    case PROP_PIXEL_ASPECT_RATIO:
+    {
+      g_free (glimage_sink->par);
+      glimage_sink->par = g_new0 (GValue, 1);
+      g_value_init (glimage_sink->par, GST_TYPE_FRACTION);
+      if (!g_value_transform (value, glimage_sink->par)) {
+        g_warning ("Could not transform string to aspect ratio");
+        gst_value_set_fraction (glimage_sink->par, 1, 1);
+      }
+      break;
+    }
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -306,6 +331,11 @@ gst_glimage_sink_finalize (GObject * object)
 
   glimage_sink = GST_GLIMAGE_SINK (object);
 
+  if (glimage_sink->par) {
+    g_free (glimage_sink->par);
+    glimage_sink->par = NULL;
+  }
+
   if (glimage_sink->caps)
     gst_caps_unref (glimage_sink->caps);
 
@@ -331,12 +361,45 @@ gst_glimage_sink_get_property (GObject * object, guint prop_id,
     case PROP_FORCE_ASPECT_RATIO:
       g_value_set_boolean (value, glimage_sink->keep_aspect_ratio);
       break;
+    case PROP_PIXEL_ASPECT_RATIO:
+      if (!glimage_sink->par) {
+        glimage_sink->par = g_new0 (GValue, 1);
+        g_value_init (glimage_sink->par, GST_TYPE_FRACTION);
+        gst_value_set_fraction (glimage_sink->par, 1, 1);
+      }
+      if (!g_value_transform (glimage_sink->par, value))
+        g_warning ("Could not transform string to aspect ratio");
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
 
+static gboolean
+gst_glimage_sink_query (GstElement * element, GstQuery * query)
+{
+  GstGLImageSink *glimage_sink = GST_GLIMAGE_SINK (element);
+  gboolean res = FALSE;
+
+  switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_CUSTOM:
+    {
+      GstStructure *structure = gst_query_get_structure (query);
+      gst_structure_set (structure, "gstgldisplay", G_TYPE_POINTER,
+          glimage_sink->display, NULL);
+      res = GST_ELEMENT_CLASS (parent_class)->query (element, query);
+      break;
+    }
+    default:
+      res = GST_ELEMENT_CLASS (parent_class)->query (element, query);
+      break;
+  }
+
+  return res;
+}
+
+
 /*
  * GstElement methods
  */
@@ -355,6 +418,24 @@ gst_glimage_sink_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_NULL_TO_READY:
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
+      if (!glimage_sink->display) {
+        gboolean ok = FALSE;
+        glimage_sink->display = gst_gl_display_new ();
+
+        /* init opengl context */
+        ok = gst_gl_display_create_context (glimage_sink->display, 0);
+        if (!ok) {
+          GST_ELEMENT_ERROR (glimage_sink, RESOURCE, NOT_FOUND,
+              GST_GL_DISPLAY_ERR_MSG (glimage_sink->display), (NULL));
+
+          if (glimage_sink->display) {
+            g_object_unref (glimage_sink->display);
+            glimage_sink->display = NULL;
+          }
+
+          return GST_STATE_CHANGE_FAILURE;
+        }
+      }
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
@@ -370,11 +451,24 @@ gst_glimage_sink_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
-      gst_glimage_sink_stop (GST_BASE_SINK (glimage_sink));
+    {
+      if (glimage_sink->stored_buffer) {
+        gst_buffer_unref (GST_BUFFER_CAST (glimage_sink->stored_buffer));
+        glimage_sink->stored_buffer = NULL;
+      }
+      if (glimage_sink->display) {
+        g_object_unref (glimage_sink->display);
+        glimage_sink->display = NULL;
+      }
+
+      glimage_sink->window_id = 0;
+      //but do not reset glimage_sink->new_window_id
+
       glimage_sink->fps_n = 0;
       glimage_sink->fps_d = 1;
       GST_VIDEO_SINK_WIDTH (glimage_sink) = 0;
       GST_VIDEO_SINK_HEIGHT (glimage_sink) = 0;
+    }
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       break;
@@ -385,44 +479,6 @@ gst_glimage_sink_change_state (GstElement * element, GstStateChange transition)
   return ret;
 }
 
-/*
- * GstBaseSink methods
- */
-
-static gboolean
-gst_glimage_sink_start (GstBaseSink * bsink)
-{
-  //GstGLImageSink* glimage_sink = GST_GLIMAGE_SINK (bsink);
-
-  GST_DEBUG ("start");
-
-  return TRUE;
-}
-
-static gboolean
-gst_glimage_sink_stop (GstBaseSink * bsink)
-{
-  GstGLImageSink *glimage_sink;
-
-  GST_DEBUG ("stop");
-
-  glimage_sink = GST_GLIMAGE_SINK (bsink);
-
-  if (glimage_sink->stored_buffer) {
-    gst_buffer_unref (GST_BUFFER_CAST (glimage_sink->stored_buffer));
-    glimage_sink->stored_buffer = NULL;
-  }
-  if (glimage_sink->display) {
-    g_object_unref (glimage_sink->display);
-    glimage_sink->display = NULL;
-  }
-
-  glimage_sink->window_id = 0;
-  //but do not reset glimage_sink->new_window_id
-
-  return TRUE;
-}
-
 static void
 gst_glimage_sink_get_times (GstBaseSink * bsink, GstBuffer * buf,
     GstClockTime * start, GstClockTime * end)
@@ -454,6 +510,8 @@ gst_glimage_sink_set_caps (GstBaseSink * bsink, GstCaps * caps)
   gboolean ok;
   gint fps_n, fps_d;
   gint par_n, par_d;
+  gint display_par_n, display_par_d;
+  guint display_ratio_num, display_ratio_den;
   GstVideoFormat format;
   GstStructure *structure;
   gboolean is_gl;
@@ -471,17 +529,76 @@ gst_glimage_sink_set_caps (GstBaseSink * bsink, GstCaps * caps)
   } else {
     is_gl = FALSE;
     ok = gst_video_format_parse_caps (caps, &format, &width, &height);
+
+    if (!ok)
+      return FALSE;
+
+    /* init colorspace conversion if needed */
+    ok = gst_gl_display_init_upload (glimage_sink->display, format,
+        width, height, width, height);
+
+    if (!ok) {
+      GST_ELEMENT_ERROR (glimage_sink, RESOURCE, NOT_FOUND,
+          GST_GL_DISPLAY_ERR_MSG (glimage_sink->display), (NULL));
+      return FALSE;
+    }
   }
+
+  gst_gl_display_set_client_reshape_callback (glimage_sink->display,
+      glimage_sink->clientReshapeCallback);
+
+  gst_gl_display_set_client_draw_callback (glimage_sink->display,
+      glimage_sink->clientDrawCallback);
+
+  gst_gl_display_set_client_data (glimage_sink->display,
+      glimage_sink->client_data);
+
   ok &= gst_video_parse_caps_framerate (caps, &fps_n, &fps_d);
   ok &= gst_video_parse_caps_pixel_aspect_ratio (caps, &par_n, &par_d);
 
   if (!ok)
     return FALSE;
 
+  /* get display's PAR */
+  if (glimage_sink->par) {
+    display_par_n = gst_value_get_fraction_numerator (glimage_sink->par);
+    display_par_d = gst_value_get_fraction_denominator (glimage_sink->par);
+  } else {
+    display_par_n = 1;
+    display_par_d = 1;
+  }
+
+  ok = gst_video_calculate_display_ratio (&display_ratio_num,
+      &display_ratio_den, width, height, par_n, par_d, display_par_n,
+      display_par_d);
+
+  if (!ok)
+    return FALSE;
+
+  if (height % display_ratio_den == 0) {
+    GST_DEBUG ("keeping video height");
+    glimage_sink->window_width = (guint)
+        gst_util_uint64_scale_int (height, display_ratio_num,
+        display_ratio_den);
+    glimage_sink->window_height = height;
+  } else if (width % display_ratio_num == 0) {
+    GST_DEBUG ("keeping video width");
+    glimage_sink->window_width = width;
+    glimage_sink->window_height = (guint)
+        gst_util_uint64_scale_int (width, display_ratio_den, display_ratio_num);
+  } else {
+    GST_DEBUG ("approximating while keeping video height");
+    glimage_sink->window_width = (guint)
+        gst_util_uint64_scale_int (height, display_ratio_num,
+        display_ratio_den);
+    glimage_sink->window_height = height;
+  }
+  GST_DEBUG ("scaling to %dx%d",
+      glimage_sink->window_width, glimage_sink->window_height);
+
   GST_VIDEO_SINK_WIDTH (glimage_sink) = width;
   GST_VIDEO_SINK_HEIGHT (glimage_sink) = height;
   glimage_sink->is_gl = is_gl;
-  glimage_sink->format = format;
   glimage_sink->width = width;
   glimage_sink->height = height;
   glimage_sink->fps_n = fps_n;
@@ -509,40 +626,9 @@ gst_glimage_sink_render (GstBaseSink * bsink, GstBuffer * buf)
   if (glimage_sink->is_gl) {
     //increment gl buffer ref before storage
     gl_buffer = GST_GL_BUFFER (gst_buffer_ref (buf));
-
-    //if glimagesink has not the display yet
-    if (glimage_sink->display == NULL) {
-      glimage_sink->display = g_object_ref (gl_buffer->display);
-
-      gst_gl_display_set_client_reshape_callback (glimage_sink->display,
-          glimage_sink->clientReshapeCallback);
-
-      gst_gl_display_set_client_draw_callback (glimage_sink->display,
-          glimage_sink->clientDrawCallback);
-    }
   }
   //is not gl
   else {
-    //if glimagesink has not the display yet
-    if (glimage_sink->display == NULL) {
-      //create a display
-      glimage_sink->display = gst_gl_display_new ();
-
-      //init opengl context
-      gst_gl_display_create_context (glimage_sink->display,
-          glimage_sink->width, glimage_sink->height, 0);
-
-      //init colorspace conversion if needed
-      gst_gl_display_init_upload (glimage_sink->display, glimage_sink->format,
-          glimage_sink->width, glimage_sink->height,
-          glimage_sink->width, glimage_sink->height);
-
-      gst_gl_display_set_client_reshape_callback (glimage_sink->display,
-          glimage_sink->clientReshapeCallback);
-
-      gst_gl_display_set_client_draw_callback (glimage_sink->display,
-          glimage_sink->clientDrawCallback);
-    }
     //blocking call
     gl_buffer = gst_gl_buffer_new (glimage_sink->display,
         glimage_sink->width, glimage_sink->height);
@@ -559,7 +645,6 @@ gst_glimage_sink_render (GstBaseSink * bsink, GstBuffer * buf)
     gst_gl_display_set_window_id (glimage_sink->display,
         glimage_sink->window_id);
   }
-
   //the buffer is cleared when an other comes in
   if (glimage_sink->stored_buffer) {
     gst_buffer_unref (GST_BUFFER_CAST (glimage_sink->stored_buffer));
@@ -571,26 +656,31 @@ gst_glimage_sink_render (GstBaseSink * bsink, GstBuffer * buf)
   //redisplay opengl scene
   if (gl_buffer->texture &&
       gst_gl_display_redisplay (glimage_sink->display,
-          gl_buffer->texture, gl_buffer->width, gl_buffer->height, 
+          gl_buffer->texture, gl_buffer->width, gl_buffer->height,
+          glimage_sink->window_width, glimage_sink->window_height,
           glimage_sink->keep_aspect_ratio))
     return GST_FLOW_OK;
-  else
-    return GST_FLOW_UNEXPECTED;
+  else {
+    GST_ELEMENT_ERROR (glimage_sink, RESOURCE, NOT_FOUND,
+        GST_GL_DISPLAY_ERR_MSG (glimage_sink->display), (NULL));
+    return GST_FLOW_ERROR;
+  }
 }
 
 
 static void
 gst_glimage_sink_xoverlay_init (GstXOverlayClass * iface)
 {
-  iface->set_xwindow_id = gst_glimage_sink_set_xwindow_id;
+  iface->set_window_handle = gst_glimage_sink_set_window_handle;
   iface->expose = gst_glimage_sink_expose;
 }
 
 
 static void
-gst_glimage_sink_set_xwindow_id (GstXOverlay * overlay, gulong window_id)
+gst_glimage_sink_set_window_handle (GstXOverlay * overlay, guintptr id)
 {
   GstGLImageSink *glimage_sink = GST_GLIMAGE_SINK (overlay);
+  gulong window_id = (gulong) id;
 
   g_return_if_fail (GST_IS_GLIMAGE_SINK (overlay));
 
@@ -607,15 +697,15 @@ gst_glimage_sink_expose (GstXOverlay * overlay)
 
   //redisplay opengl scene
   if (glimage_sink->display && glimage_sink->window_id) {
-    
+
     if (glimage_sink->window_id != glimage_sink->new_window_id) {
       glimage_sink->window_id = glimage_sink->new_window_id;
       gst_gl_display_set_window_id (glimage_sink->display,
           glimage_sink->window_id);
     }
 
-    gst_gl_display_redisplay (glimage_sink->display, 0, 0, 0, 
-      glimage_sink->keep_aspect_ratio);
+    gst_gl_display_redisplay (glimage_sink->display, 0, 0, 0, 0, 0,
+        glimage_sink->keep_aspect_ratio);
   }
 }
 
@@ -624,6 +714,8 @@ static gboolean
 gst_glimage_sink_interface_supported (GstImplementsInterface * iface,
     GType type)
 {
+  if (type != GST_TYPE_X_OVERLAY)
+    return FALSE;
   return TRUE;
 }
 
