@@ -48,29 +48,46 @@
 #include "gstglimagesink.h"
 
 #include "gstglfiltercube.h"
+#include "gstgleffects.h"
 
 GType gst_gl_filter_cube_get_type (void);
+GType gst_gl_effects_get_type (void);
 
 #ifndef OPENGL_ES2
 #include "gstgltestsrc.h"
 #include "gstglfilterlaplacian.h"
 #include "gstglfilterglass.h"
 #include "gstglfilterapp.h"
+#include "gstglfilterreflectedscreen.h"
+#include "gstglfiltershader.h"
 #include "gstglcolorscale.h"
 #include "gstgldeinterlace.h"
-#include "gstgleffects.h"
-#include "gstglbumper.h"
+#include "gstglmosaic.h"
 
-GType gst_gl_effects_get_type (void);
 GType gst_gl_deinterlace_get_type (void);
 GType gst_gl_filter_app_get_type (void);
+GType gst_gl_filter_reflected_screen_get_type (void);
 GType gst_gl_filterblur_get_type (void);
+GType gst_gl_filtershader_get_type (void);
 GType gst_gl_filtersobel_get_type (void);
 GType gst_gl_filter_laplacian_get_type (void);
 GType gst_gl_filter_glass_get_type (void);
-GType gst_gl_overlay_get_type (void);
+GType gst_gl_mosaic_get_type (void);
+
+#ifdef HAVE_PNG
+#include "gstgldifferencematte.h"
+#include "gstglbumper.h"
+
 GType gst_gl_differencematte_get_type (void);
 GType gst_gl_bumper_get_type (void);
+
+#ifdef HAVE_JPEG
+#include "gstgloverlay.h"
+
+GType gst_gl_overlay_get_type (void);
+
+#endif
+#endif
 #endif
 
 #define GST_CAT_DEFAULT gst_gl_gstgl_debug
@@ -102,35 +119,26 @@ plugin_init (GstPlugin * plugin)
     return FALSE;
   }
 
+  if (!gst_element_register (plugin, "gleffects",
+          GST_RANK_NONE, gst_gl_effects_get_type ())) {
+    return FALSE;
+  }
 #ifndef OPENGL_ES2
   if (!gst_element_register (plugin, "gltestsrc",
           GST_RANK_NONE, GST_TYPE_GL_TEST_SRC)) {
     return FALSE;
   }
 
-  if (!gst_element_register (plugin, "gloverlay",
-          GST_RANK_NONE, gst_gl_overlay_get_type ())) {
-    return FALSE;
-  }
-  if (!gst_element_register (plugin, "gldifferencematte",
-          GST_RANK_NONE, gst_gl_differencematte_get_type ())) {
-    return FALSE;
-  }
-  if (!gst_element_register (plugin, "glbumper",
-          GST_RANK_NONE, gst_gl_bumper_get_type ())) {
-    return FALSE;
-  }
-
-  if (!gst_element_register (plugin, "gleffects",
-          GST_RANK_NONE, gst_gl_effects_get_type ())) {
-    return FALSE;
-  }
-
   if (!gst_element_register (plugin, "glfilterblur",
           GST_RANK_NONE, gst_gl_filterblur_get_type ())) {
     return FALSE;
   }
 
+  if (!gst_element_register (plugin, "glshader",
+          GST_RANK_NONE, gst_gl_filtershader_get_type ())) {
+    return FALSE;
+  }
+
   if (!gst_element_register (plugin, "glfiltersobel",
           GST_RANK_NONE, gst_gl_filtersobel_get_type ())) {
     return FALSE;
@@ -151,6 +159,11 @@ plugin_init (GstPlugin * plugin)
     return FALSE;
   }
 
+  if (!gst_element_register (plugin, "glfilterreflectedscreen",
+          GST_RANK_NONE, GST_TYPE_GL_FILTER_REFLECTED_SCREEN)) {
+    return FALSE;
+  }
+
   if (!gst_element_register (plugin, "gldeinterlace",
           GST_RANK_NONE, GST_TYPE_GL_DEINTERLACE)) {
     return FALSE;
@@ -160,6 +173,28 @@ plugin_init (GstPlugin * plugin)
           GST_RANK_NONE, GST_TYPE_GL_COLORSCALE)) {
     return FALSE;
   }
+
+  if (!gst_element_register (plugin, "glmosaic",
+          GST_RANK_NONE, GST_TYPE_GL_MOSAIC)) {
+    return FALSE;
+  }
+#ifdef HAVE_PNG
+  if (!gst_element_register (plugin, "gldifferencematte",
+          GST_RANK_NONE, gst_gl_differencematte_get_type ())) {
+    return FALSE;
+  }
+
+  if (!gst_element_register (plugin, "glbumper",
+          GST_RANK_NONE, gst_gl_bumper_get_type ())) {
+    return FALSE;
+  }
+#ifdef HAVE_JPEG
+  if (!gst_element_register (plugin, "gloverlay",
+          GST_RANK_NONE, gst_gl_overlay_get_type ())) {
+    return FALSE;
+  }
+#endif
+#endif
 #endif
 
   return TRUE;
